/*
 * commandline.c - Parse and process possible command line options
 *
 * Copyright (C) 2003,2004, 2006, 2007 Brailcom, o.p.s.
 * Copyright (C) 2010 OpenTTS developers
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "config.h"

#include <assert.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "commandline.h"
#include "options.h"

static struct option long_options[] = {
	{"run-daemon", 0, 0, 'd'},
	{"run-single", 0, 0, 's'},
	{"log-level", 1, 0, 'l'},
	{"log-path", 1, 0, 'L'},
	{"config-file", 1, 0, 'C'},
	{"device", 1, 0, 'D'},
	{"coding", 1, 0, 'c'},
	{"language", 1, 0, 'i'},
	{"synthesis", 1, 0, 'S'},
	{"dont-init-tables", 0, 0, 't'},
	{"probe", 0, 0, 'p'},
	{"version", 0, 0, 'v'},
	{"help", 0, 0, 0},
	{0, 0, 0, 0}
};

static char *short_options = "dsvhpti:l:L:C:D:S:c:";

static int option_to_int(char *arg)
{
	int rc = 0;
	char *tmp;
	long val;

	val = strtol(arg, &tmp, 10);
	if (tmp != arg)
		rc = val;
	return rc;
}

static void print_help(char *argv[])
{
	assert(argv);
	assert(argv[0]);

	printf
	    ("Usage: %s [-{d|s}] [-l {1|2|3|4|5}] [-L=logfile] [-c=encoding] | [-v] | [-h]\n",
	     argv[0]);
	printf
	    ("OSpeakup -- Interface between OpenTTSD and SpeakUp (GNU GPL)\n\n");
	printf("-d, --run-daemon     -      Run as a daemon\n");
	printf("-s, --run-single     -      Run as single application\n");
	printf("-l, --log-level      -      Set log level (1..5)\n");
	printf("-L, --log-path       -      Set path for ospeakup.log\n");
	printf
	    ("-D, --device         -      Specify the device name of Speakup software synthesis\n");
	printf
	    ("-i, --language       -      Set default language for speech output\n");
	printf
	    ("-c, --coding         -      Specify the default encoding to use\n");
	printf
	    ("-t, --dont-init-tables -    Don't rewrite /sys tables for optimal software synthesis\n");
	printf
	    ("-p, --probe          -      Initialize everything and try to say some message\n");
	printf
	    ("                            but don't connect to SpeakUp. For testing purposes.\n");
	printf("-v, --version        -      Report version of this program\n");
	printf("-h, --help           -      Print this info\n\n");
	printf("Copyright (C) 2003,2005 Brailcom, o.p.s.\n");
	printf("Copyright (C) 2010 OpenTTS developers\n");
	printf
	    ("This is free software; you can redistribute it and/or modify it\n");
	printf
	    ("under the terms of the GNU General Public License as published by\n");
	printf
	    ("the Free Software Foundation; either version 2, or (at your option)\n");
	printf("any later version. Please see COPYING for more details.\n\n");
	printf("Please report bugs to %s\n\n", PACKAGE_BUGREPORT);
}

static void print_version(void)
{
	printf("%s %s\n", PACKAGE, VERSION);
	printf("Copyright (C) 2002-2003 Brailcom, o.p.s.\n");
	printf("Copyright (C) 2010 OpenTTS developers\n");
	printf("%s comes with ABSOLUTELY NO WARRANTY.\n", PACKAGE);
	printf("You may redistribute copies of %s\n", PACKAGE);
	printf("under the terms of the GNU General Public License.\n");
	printf
	    ("For more information about these matters, see the file COPYING.\n");
}

void parse_commandline(int argc, char **argv)
{
	int opt;
	int opt_index;
	int val;

	assert(argc > 0);
	assert(argv);

	while (1) {
		opt_index = 0;

		opt =
		    getopt_long(argc, argv, short_options, long_options,
				&opt_index);
		if (opt == -1)
			break;

		switch (opt) {
		case 'd':
			set_int_option(&options.spd_spk_mode, MODE_DAEMON,
				       COMMAND_LINE);
			break;
		case 's':
			set_int_option(&options.spd_spk_mode, MODE_SINGLE,
				       COMMAND_LINE);
			break;
		case 'l':
			val = option_to_int(optarg);
			if ((val >= 1) && (val <= 5)) {
				set_int_option(&options.log_level, val,
					       COMMAND_LINE);
			} else {
				printf
				    ("Invalid log level (must be between 1 and 5)\n");
				exit(1);
			}
			break;
		case 'L':
			set_string_option(&options.log_path, optarg,
					  COMMAND_LINE);
			break;
		case 'C':
			set_string_option(&options.config_file_name, optarg,
					  COMMAND_LINE);
			break;
		case 'D':
			set_string_option(&options.speakup_device, optarg,
					  COMMAND_LINE);
			break;
		case 'c':
			set_string_option(&options.speakup_coding, optarg,
					  COMMAND_LINE);
			break;
		case 'i':
			set_string_option(&options.language, optarg,
					  COMMAND_LINE);
		case 'v':
			print_version();
			exit(0);
			break;
		case 'h':
			print_help(argv);
			exit(0);
			break;
		case 'p':
			set_int_option(&options.probe_mode, 1, COMMAND_LINE);
			break;
		case 't':
			set_int_option(&options.dont_init_tables, 1,
				       COMMAND_LINE);
			break;
		default:
			printf("Error: Unrecognized option\n\n");
			print_help(argv);
			exit(1);
		}
	}
}
