/*
 * configuration.c -- Dotconf configuration for ospeakup
 *
 * This is a complete rewrite of the configuration handler for
 * speechd-up, which was Copyright (C) 2001, 2002, 2003, 2006 Brailcom, o.p.s.
 *
 * Copyright (C) 2010 OpenTTS developers
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dotconf.h>

#include "configuration.h"
#include "log.h"
#include "options.h"

/*
 * Declare the dotconf error handler and callbacks.
 */
static FUNC_ERRORHANDLER(errorhandler);
static DOTCONF_CB(cb_dontInitTables);
static DOTCONF_CB(cb_language);
static DOTCONF_CB(cb_logLevel);
static DOTCONF_CB(cb_logPath);
static DOTCONF_CB(cb_speakupCharacters);
static DOTCONF_CB(cb_speakupChartab);
static DOTCONF_CB(cb_speakupCoding);
static DOTCONF_CB(cb_speakupDevice);

/*
 * Initialize the array of configuration options.
 */
static const configoption_t configOptions[] = {
	{"DontInitTables", ARG_TOGGLE, cb_dontInitTables, NULL, CTX_ALL,},
	{"Language", ARG_STR, cb_language, NULL, CTX_ALL,},
	{"LogLevel", ARG_INT, cb_logLevel, NULL, CTX_ALL,},
	{"LogPath", ARG_STR, cb_logPath, NULL, CTX_ALL,},
	{"SpeakupCharacters", ARG_STR, cb_speakupCharacters, NULL, CTX_ALL,},
	{"SpeakupChartab", ARG_STR, cb_speakupChartab, NULL, CTX_ALL,},
	{"SpeakupCoding", ARG_STR, cb_speakupCoding, NULL, CTX_ALL,},
	{"SpeakupDevice", ARG_STR, cb_speakupDevice, NULL, CTX_ALL,},
	LAST_OPTION
};

static FUNC_ERRORHANDLER(errorhandler)
{
	LOG(2, "Configuration file error: %s\n", msg);
	return 0;
}

static DOTCONF_CB(cb_dontInitTables)
{
	if (options.dont_init_tables.source != COMMAND_LINE) {
		LOG(3, "setting %s to %i\n", cmd->name, cmd->data.value);
		set_int_option(&options.dont_init_tables, cmd->data.value,
			       CONFIG_FILE);
		LOG(3, "setting %s has value %i\n", cmd->name, cmd->data.value);
	}
	return NULL;
}

static DOTCONF_CB(cb_language)
{
	assert(cmd->data.str);
	if (options.language.source != COMMAND_LINE) {
		LOG(3, "setting %s to %s\n", cmd->name, cmd->data.str);
		set_string_option(&options.language, cmd->data.str,
				  CONFIG_FILE);
		LOG(3, "setting %s has value %s\n", cmd->name, cmd->data.str);
	}
	return NULL;
}

static DOTCONF_CB(cb_logLevel)
{
	if ((cmd->data.value < 1) || (cmd->data.value > 5))
		FATAL(-1, "Log level must be between 1 and 5");
	if (options.log_level.source != COMMAND_LINE) {
		LOG(3, "setting %s to %i\n", cmd->name, cmd->data.value);
		set_int_option(&options.log_level, cmd->data.value,
			       CONFIG_FILE);
		LOG(3, "setting %s has value %i\n", cmd->name, cmd->data.value);
	}
	return NULL;
}

static DOTCONF_CB(cb_logPath)
{
	assert(cmd->data.str);
	if (options.log_path.source != COMMAND_LINE) {
		LOG(3, "setting %s to %s\n", cmd->name, cmd->data.str);
		set_string_option(&options.log_path, cmd->data.str,
				  CONFIG_FILE);
		LOG(3, "setting %s has value %s\n", cmd->name, cmd->data.str);
	}
	return NULL;
}

static DOTCONF_CB(cb_speakupCharacters)
{
	assert(cmd->data.str);
	if (options.speakup_characters.source != COMMAND_LINE) {
		LOG(3, "setting %s to %s\n", cmd->name, cmd->data.str);
		set_string_option(&options.speakup_characters, cmd->data.str,
				  CONFIG_FILE);
		LOG(3, "setting %s has value %s\n", cmd->name, cmd->data.str);
	}
	return NULL;
}

static DOTCONF_CB(cb_speakupChartab)
{
	assert(cmd->data.str);
	if (options.speakup_chartab.source != COMMAND_LINE) {
		LOG(3, "setting %s to %s\n", cmd->name, cmd->data.str);
		set_string_option(&options.speakup_chartab, cmd->data.str,
				  CONFIG_FILE);
		LOG(3, "setting %s has value %s\n", cmd->name, cmd->data.str);
	}
	return NULL;
}

static DOTCONF_CB(cb_speakupCoding)
{
	assert(cmd->data.str);
	if (options.speakup_coding.source != COMMAND_LINE) {
		LOG(3, "setting %s to %s\n", cmd->name, cmd->data.str);
		set_string_option(&options.speakup_coding, cmd->data.str,
				  CONFIG_FILE);
		LOG(3, "setting %s has value %s\n", cmd->name, cmd->data.str);
	}
	return NULL;
}

static DOTCONF_CB(cb_speakupDevice)
{
	assert(cmd->data.str);
	if (options.speakup_device.source != COMMAND_LINE) {
		LOG(3, "setting %s to %s\n", cmd->name, cmd->data.str);
		set_string_option(&options.speakup_device, cmd->data.str,
				  CONFIG_FILE);
		LOG(3, "setting %s has value %s\n", cmd->name, cmd->data.str);
	}
	return NULL;
}

void load_configuration(void)
{
	configfile_t *configfile;

	configfile =
	    dotconf_create(options.config_file_name.value, configOptions, NULL,
			   CASE_INSENSITIVE);
	if (!configfile)
		exit(-1);
	configfile->errorhandler = (dotconf_errorhandler_t) errorhandler;
	if (dotconf_command_loop(configfile) == 0)
		FATAL(-1, "Error reading config file\n");
	dotconf_cleanup(configfile);
	LOG(1, "Configuration has been read from \"%s\"",
	    options.config_file_name.value);
}
